# coding=UTF-8

import pymysql
from os import path
import settings
from settings import KONFIGURACJA as conf
from botconsts import xml_acc, xml_player

class AIO(object):
	"""Abstrakcyjna klasa IO"""
	def __init__(self):
		pass
	def getPlayer(self, name):
		pass

	def isAccountFree(self, name):
		pass

	def isNickFree(self, name):
		pass

	def addPlayer(self, accName, passwd, playerName, vocation=conf['defprof'], town=conf['defcity']):
		pass

class XmlIO(AIO):
	"""Klasa IO dla XML"""
	def __init__(self):
		self.accounts = path.join(settings.DANE_OTS[0], 'accounts')
		self.players = path.join(settings.DANE_OTS[0], 'players')
		if settings.DEBUG:
			print self.accounts, self.players

	def getPlayer(self, name):
		return None

	def isAccountFree(self, name):
		return not path.isfile(path.join(self.accounts,name+'.xml'))

	def isNickFree(self, name):
		return not path.isfile(path.join(self.players,name+'.xml'))

	def addPlayer(self, accName, passwd, playerName, vocation=conf['defprof'], town=conf['defcity']):
		acc = xml_acc % (passwd, playerName)
		player = xml_player % (playerName,
								int(accName),
								vocation,
								conf['postac']['lvl'],
		 						conf['postac']['cap'],
								conf['postac']['mlv'],
								settings.KONFIGURACJA_XML[town]['temple_x'],
								settings.KONFIGURACJA_XML[town]['temple_y'],
								settings.KONFIGURACJA_XML[town]['temple_z'],
								settings.KONFIGURACJA_XML[town]['temple_x'],
								settings.KONFIGURACJA_XML[town]['temple_y'],
								settings.KONFIGURACJA_XML[town]['temple_z'],
								conf['postac']['health'],
								conf['postac']['health'],
								conf['postac']['mana'],
								conf['postac']['mana'],
								conf['postac']['look']['type'],
								conf['postac']['look']['head'],
								conf['postac']['look']['body'],
								conf['postac']['look']['legs'],
								conf['postac']['look']['feet'])

		acc_f = open(path.join(self.accounts,accName+'.xml'),'w')
		acc_f.write(acc)
		acc_f.close()
		player_f = open(path.join(self.players,playerName+'.xml'),'w')
		player_f.write(player)
		player_f.close()

class DbIO(AIO):
	"""Klasa IO dla mysql"""
	def __init__(self):
		print u'>Łączenie z bazą danych'
		self.db = pymysql.connect(host=settings.DANE_BAZY[0], user=settings.DANE_BAZY[1],
								  passwd=settings.DANE_BAZY[2], db=settings.DANE_BAZY[3])
		
	def getPlayer(self, name):
		curr = self.db.cursor()
		curr.execute('SELECT `name`, `level`, `online`, `lastlogout` FROM `players` WHERE name = %s LIMIT 1',name)
		ret = curr.fetchone()
		curr.close()
		return ret

	def isAccountFree(self, name):
		curr = self.db.cursor()
		curr.execute('SELECT `id` FROM `accounts` WHERE `name` = %s',name)
		ret = curr.fetchone()
		curr.close()
		return ret == None

	def isNickFree(self, name):
		curr = self.db.cursor()
		curr.execute('SELECT `id` FROM `players` WHERE `name` = %s',name)
		ret = curr.fetchone()
		curr.close()
		return ret == None

	def addPlayer(self, accName, passwd, playerName, vocation=conf['defprof'], town=conf['defcity']):
		curr = self.db.cursor()
		curr.execute("INSERT INTO `accounts` (`name`, `password`) VALUES (%s, %s)", (accName, passwd))
		curr.execute("SELECT LAST_INSERT_ID()")

		curr.execute("INSERT INTO `players` (`name`, `account_id`,`level`, `maglevel`, `health`, `healthmax`, `mana`, `manamax`, `promotion`, `looktype`, `lookhead`, `lookbody`, `looklegs`, `lookfeet`, `lookaddons`, `vocation`, `town_id`) VALUES (%s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s)",
		(playerName,
		 curr.fetchone()[0],
		 conf['postac']['lvl'],
		 conf['postac']['mlv'],
		 conf['postac']['health'],
		 conf['postac']['health'],
		 conf['postac']['mana'],
		 conf['postac']['mana'],
		 conf['postac']['promo'],
		 conf['postac']['look']['type'],
		 conf['postac']['look']['head'],
		 conf['postac']['look']['body'],
		 conf['postac']['look']['legs'],
		 conf['postac']['look']['feet'],
		 conf['postac']['look']['addons'],
		 vocation,
		 town)
		)
		if settings.DEBUG:
			curr.execute("SELECT LAST_INSERT_ID()")
			print curr.fetchone()[0]
		self.db.commit()
		curr.close()