# coding=UTF-8

import settings
import re
from serverio import DbIO, XmlIO
from datetime import datetime
from pygglib2.Helpers import Enum
from settings import KONFIGURACJA as conf
import types

DB_ENABLE = True
if not settings.DEBUG:
	DB_ENABLE = True

TS = Enum({
	'free':0,
	'name':1,
	'passwd':2,
	'nick':3,
	'voc':4,
	'city':5,
	'end':6,
})


class Validator(object):
	def __init__(self):
		self.passPatt = re.compile(r'^[a-zA-Z0-9]*$')
		self.accPatt = re.compile(r'^[1-9][0-9]*$')
		self.nickPatt = re.compile(r'^[a-zA-Z ]*$')
		
	def isAccValid(self, msg):
		if settings.TYP_OTS == 'mysql':
			return self.passPatt.match(msg) != None
		elif settings.TYP_OTS == 'xml':
			return self.accPatt.match(msg) != None

	def isPassValid(self, msg):
		return self.passPatt.match(msg) != None
	
	def isNickValid(self, msg):
		return self.nickPatt.match(msg) != None

class Person(object):
	uin = 0
	talkState = 0

class Talker(object):
	persons = {}
	val = Validator()

	def __init__(self):
		self.words = {}
		if settings.TYP_OTS == 'mysql':
			self.db = DbIO()
		elif settings.TYP_OTS == 'xml':
			self.db = XmlIO()
			
		for t in settings.WORDS:
			for elem in t[0]:
				self.words[elem.lower()] = t[1]

	def create(self, uin, msg):
		self.persons[uin].talkState = TS.name
		return u'\nTworzenie postaci możesz w każdej chwili przerwać wpisując "anuluj!"\n' \
			   + u'Podaj numer konta' if settings.TYP_OTS == 'xml' else u'Podaj nazwę konta'


	def talk(self, person, msg):
		if person.talkState == TS.name:
			if settings.TYP_OTS == 'xml':
				maxLen = 8
			else:
				maxLen = 30
			if self.val.isAccValid(msg) and 5<len(msg)<maxLen:
				if not self.db.isAccountFree(msg):
					return u'Numer konta jest już zajęty, podaj inny' if settings.TYP_OTS == 'xml' else u'Nazwa konta jest już zajęta, podaj inną'
				person.name = msg
				person.talkState = TS.passwd
				return u'Ok, podaj haslo.'
			else:
				return u'Numer konta jest nieprawidłowy! Podaj jeszcze raz.' if settings.TYP_OTS == 'xml' else u'Nazwa konta jest nieprawidłowa! Podaj jeszcze raz.'
		elif person.talkState == TS.passwd:
			if self.val.isPassValid(msg) and 5<len(msg)<30:
				person.passwd = msg
				person.talkState = TS.nick
				return u'Ok, podaj nick.'
			else:
				return u'Hasło jest nieprawidłowe! Podaj jeszcze raz.'
		elif person.talkState == TS.nick and 2<len(msg)<20:
			if self.val.isNickValid(msg):
				if not self.db.isNickFree(msg):
					return u'Nick jest już zajęty! Podaj jeszcze raz.'
				person.nick = msg
				person.talkState = TS.end
				if conf['wybor_profesji']:
					person.talkState = TS.voc
					return u'Wybierz jedną z profesji: %s' % ', '.join([x for x in conf['profesje']])
				elif conf['wybor_miasta']:
					person.talkState = TS.city
					return u'Wybierz jedno z miast:\n%s' % ', '.join([
							'%s - %s\n' % (x, x[x][1]) for x in conf['miasta']
					])
				else:
					person.talkState = TS.end
			else:
				return u'Nieprawidłowy nick! Podaj jeszcze raz'
		elif person.talkState == TS.voc:
			#sprawdzenie czy jest taka profesja
			if msg.lower() in [x.lower() for x in conf['profesje']]:
				#nowy słownik {vocation_lowcase:voc_id}, od razu ustawiamy voc_id
				person.voc = {x.lower():conf['profesje'][x]
							  for x in conf['profesje']
							 }[msg.lower()]
				if conf['wybor_miasta']:
					person.talkState = TS.city
					return u'Wybierz jedno z miast:\n%s' % '\n'.join([
							'%s - %s' % (x, conf['miasta'][x][1]) for x in conf['miasta']
					])
				else:
					person.talkState = TS.end
			else:
				return u'Profesja nieprawidłowa, podaj jeszcze raz.'
		elif person.talkState == TS.city:
			#sprawdzenie czy jest takie miasto
			if msg.lower() in [x.lower() for x in conf['miasta']]:
				#nowy słownik {nazwa_miasta_lowcase:town_id}, od razu ustawiamy town_id
				person.city = {x.lower():conf['miasta'][x][0]
							   for x in conf['miasta']
							  }[msg.lower()]
				person.talkState = TS.end
			else:
				return u'Miasto nieprawidłowe, podaj jeszcze raz.'


		if person.talkState == TS.end:
			if DB_ENABLE:
				person.talkState = TS.free
				if not conf['wybor_miasta']:
					person.city = conf['defcity']
				if not conf['wybor_profesji']:
					person.voc = conf['defprof']
				if self.db.isNickFree(person.nick) and self.db.isAccountFree(person.name):
					self.db.addPlayer(person.name, person.passwd, person.nick, vocation=person.voc, town=person.city)
				else:
					return u"Stało się coś co stać się nigdy nie powinno... Zacznij od nowa ;)"
			if settings.DEBUG:
				return "Acc: %s\nPass: %s\nNick: %s\nVoc: %d\nTown: %d" % (person.name, person.passwd, person.nick, person.voc, person.city)
			return u'Ok! Teraz możesz się zalogować do gry :)'


		return u'Anulowano'

	def getRespond(self, uin, msg):
		if uin in self.persons:
			if self.persons[uin].talkState != TS.free:
				if msg.lower() == 'anuluj!':
					self.persons[uin].talkState = TS.free
				return self.talk(self.persons[uin],msg)
		else:
			p = Person()
			p.uin = uin
			self.persons[uin] = p
			if settings.DEBUG:
				print "Dodano",uin

		com = msg.split(' ', 1)
		if com[0].lower() in self.words:
			if type(self.words[com[0]]) == types.StringType:
				attr = getattr(self, self.words[com[0]], None)
				if attr:
					return attr(uin, msg)
			else:
				return self.words[com[0]]
		else:
			return u'Aby uzyskać pomoc wpisz pomoc'

	def info(self,uin,msg):
		#Sprawdzanie graczy w otsach XML mogłoby 'troszkę' przymulić ;)
		if settings.TYP_OTS == 'xml':
			return u'Nie możesz sprawdzać graczy :/'

		com = msg.split(' ', 1)
		if len(com) < 2:
			return 'Podaj nick gracza'
		else:
			player = self.db.getPlayer(com[1])
			if player == None:
				return 'Nie ma takiego gracza :('
			else:
				ret = u'Gracz: %s\nPoziom: %d\n' % (player[0], player[1])
				if player[2] == 1:
					ret += u'Jest obecnie zalogowany'
				else:
					if player[3] == 0:
						ret += u'Nie był nigdy zalogowany'
					else:
						ret += u'Ostatnio zalogowany był %s' % datetime.fromtimestamp(int(player[3])).strftime('%d-%m-%Y o %H:%M')
				return ret